import React, { useEffect, useState } from "react";
import {
    MapContainer,
    Marker,
    Popup,
    TileLayer,
    Tooltip,
    useMap,
} from "react-leaflet";
import "leaflet/dist/leaflet.css";
import "leaflet-defaulticon-compatibility/dist/leaflet-defaulticon-compatibility.css";
import "leaflet-defaulticon-compatibility";
import "leaflet-contextmenu";
import "leaflet-contextmenu/dist/leaflet.contextmenu.css";
import Link from "next/link";
import { convertDate } from "../../../utils/lib/dates";
import * as pointers from "./pointerIcon";

const Map = ({ data, lat,lng,zoom }) => {
    let waypoints = [];
    if (data?.data?.length > 0) {
        data?.data?.forEach((unitat) => {
            let tmp = L.latLng(
                unitat?.geometry?.coordinates[1],
                unitat?.geometry?.coordinates[0]
            );
            waypoints.push(tmp);
        });
    }
    const getIcon = (index) => {
        let last = data?.data?.length - 1        
        if (index === 0) return `flagStart`;
        if (index+1===data?.data?.length) return `flagFinish`
        return `pointBlue`        
    };

    function Routing() {
        const map = useMap();

        useEffect(() => {
            if (!map) return;

            // create a red polyline from an array of LatLng points
            let polyline = L.polyline(waypoints, { color: "blue" }).addTo(map);

            return () => map.removeControl(polyline);
        }, [map]);

        return null;
    }

    function Markers() {
        return (
            <>
                {data?.data?.map((unitat, index) => {
                    const { geometry, properties, object } = unitat;
                    const { coordinates } = geometry;

                    return (
                        <Marker
                            icon={pointers[getIcon(index)]}
                            id={unitat?._id}
                            key={unitat?._id}
                            position={[coordinates[1], coordinates[0]]}
                            draggable={false}
                            animate={false}
                            object={object}
                        >
                            <Tooltip
                                className={`Blue`}
                                direction="top"
                                opacity={1}
                            >
                                {`${properties?.gps?.speed}km/h ${convertDate(
                                    properties?.timestamp
                                )}`}
                            </Tooltip>
                            <Popup>
                                <table>
                                    <tbody>
                                        {/* <tr>
                                                <td>
                                                    <strong>Nom:</strong>
                                                </td>
                                                <td>
                                                    {object?.name.toUpperCase()}
                                                </td>
                                            </tr>                                             */}
                                        {/* <tr>
                                                <td>
                                                    <strong>Address:</strong>
                                                </td>
                                                <td>E30, 14778, Germany</td>
                                            </tr>{" "} */}
                                        <tr>
                                            <td>
                                                <strong>Posició:</strong>
                                            </td>
                                            <td>
                                                <Link
                                                    href={`https://maps.google.com/maps?q=${coordinates[1]},${coordinates[0]}&amp;t=m`}
                                                >
                                                    <a
                                                        target="_blank"
                                                        rel="noreferrer"
                                                    >
                                                        {coordinates[1].toFixed(
                                                            5
                                                        )}
                                                        °,
                                                        {coordinates[0].toFixed(
                                                            5
                                                        )}
                                                        °
                                                    </a>
                                                </Link>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Altitud:</strong>
                                            </td>
                                            <td>
                                                {properties?.gps?.altitude} m
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Angle:</strong>
                                            </td>
                                            <td>{properties?.gps?.angle}°</td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Velocitat:</strong>
                                            </td>
                                            <td>
                                                {properties?.gps?.speed} km/h
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Data:</strong>
                                            </td>
                                            <td>
                                                {convertDate(
                                                    properties?.timestamp
                                                )}
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <strong>Senyal:</strong>
                                            </td>
                                            <td>
                                                {
                                                    properties?.ioElements
                                                        ?.gsm_strengh
                                                }
                                            </td>
                                        </tr>
                                        <tr>
                                            <td>
                                                <Link
                                                    href={`https://www.google.com/maps?q&layer=c&cbll=${coordinates[1]},${coordinates[0]}`}
                                                >
                                                    <a
                                                        target="_blank"
                                                        rel="noreferrer"
                                                    >
                                                        {"Street View"}
                                                    </a>
                                                </Link>
                                            </td>
                                        </tr>
                                    </tbody>
                                </table>
                            </Popup>
                        </Marker>
                    );
                })}
                <style jsx global>{`
                    .tipVisible {
                        border: none !important;
                        box-shadow: none !important;
                        background-color: transparent !important;
                        font-weight: 800;
                        font-size: smaller;
                    }
                    .tipVisible.Black {
                        color: black;
                    }
                    .tipVisible.Blue {
                        color: blue;
                    }
                    .leaflet-tooltip-top:before {
                        border-top-color: transparent !important;
                    }
                `}</style>
            </>
        );
    }

    return (
        <>  
            <MapContainer
                center={[data?.data[0]?.geometry?.coordinates[1] || lat , data?.data[0]?.geometry?.coordinates[0] || lng]}
                zoom={zoom}
                scrollWheelZoom={true}
            >
                <TileLayer
                    attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                    url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
                />
                <Routing />
                <Markers />
            </MapContainer>
            <style jsx>{`
                .containerMap {
                    height: 95%;
                    position: fixed;
                    width: calc(100vw);
                }

                /*Landscape & Portrait*/
                @media (max-width: 600px), (max-height: 600px) {
                    .containerMap {
                        width: calc(100vw);
                    }
                }
            `}</style>
        </>
    );
};

export default Map;
