import mongoose from "mongoose";
import { firebaseAdmin } from "../utils/auth/firebaseAdmin";

const { Schema } = mongoose;

const ObjectId = mongoose.Types.ObjectId;

const direction = new Schema({
    place_name: { type: String, default: "" },
    text: { type: String, default: "" },
    address: { type: String, default: "" },
    geometry: {
        type: { type: String, default: "Point" },
        coordinates: { type: [Number], default: [0, 0] },
    }
});

//Schema que gestiona els comptes
const accountSchema = new Schema({
    // _id: ObjectId,
    _id: { type: String, trim: true },
    idEntity: { type: ObjectId, ref: "Entity" },
    email: {
        type: String,
        index: true,
        unique: true,
        required: true,
        trim: true,
    },
    displayName: { type: String, default: "" },
    main: { type: Boolean, default: false },
    admin: { type: Boolean, default: false },
    active: { type: Boolean, default: true },
    map: {
        geometry: {
            type: { type: String, default: "Point" },
            coordinates: { type: [Number], default: [0, 0] },
        },
        zoom: { type: Number, default: 15 },
        scrollWheelZoom: { type: Boolean, default: true },
        layer: { type: String, default: "osm" },
        geocoding: { type: Boolean, default: true },
    },
    directions: {
        last: [direction],
        fav: [direction]
    },
    created: {
        type: Date,
        default: Date.now,
    },
});

export default mongoose.models.Account ||
    mongoose.model("Account", accountSchema);

export const createAccountInFirebase = ({ email, password, displayName }) => {
    return new Promise((resolve) => {
        firebaseAdmin
            .auth()
            .createUser({
                email: email,
                emailVerified: false,
                password: password,
                displayName: displayName,
                disabled: false,
            })
            .then((user) => {
                //resolve User
                resolve(user);
            })
            .catch(() => resolve(false));
    });
};

export const createAccountInMongoDB = (props) => {
    return new Promise((resolve) => {
        if (props.uid) {
            let account = new mongoose.models.Account({
                ...props,
                _id: props.uid,
            }); //Schema Mongoose
            account
                .save()
                .then((user) => resolve(user))
                .catch((e) => {
                    console.log(e);
                    resolve(false);
                });
        } else {
            resolve(false);
        }
    });
};

//Rollback usuari de Firebase si no es crea a MongoDB
export const rollBackAccountFirebase = ({ uid }) => {
    firebaseAdmin
        .auth()
        .deleteUser(uid)
        .then(() => {
            console.log("Successfully deleted user by Firebase");
        })
        .catch((error) => {
            console.log("Error deleting user firebase:", error);
        });
};

//Generar i enviar un link per a verificar el nou usuari
export const createLinkVerificationEmail = (user) => {
    return new Promise((resolve) => {
        // Admin SDK API to generate the email verification link.
        const useremail = user.email;
        firebaseAdmin
            .auth()
            .generateEmailVerificationLink(useremail /*actionCodeSettings*/)
            .then((link) => {
                resolve(link);
            })
            .catch((error) => {
                resolve(false);
            });
    });
};

//Enviar el email amb el link per verificar generat
export const sendVerificationEmail = (user, link, EMAIL_USER, EMAIL_PASS) => {
    "use strict";
    const nodemailer = require("nodemailer");

    // async..await is not allowed in global scope, must use a wrapper
    async function main() {
        // create reusable transporter object using the default SMTP transport
        const transporter = nodemailer.createTransport({
            host: "smtp.servidor-correo.net",
            port: 587,
            secure: false, // true for 465, false for other ports
            auth: {
                user: EMAIL_USER,
                pass: EMAIL_PASS,
            },
        });

        // send mail with defined transport object
        transporter
            .sendMail({
                from: '"Poltracker" <support@poltracker.es>', // sender address
                to: user.email, // list of receivers
                subject:
                    "Verifica l'adreça electrònica de l'aplicació Poltracker", // Subject line
                //text: "Hello world?", // plain text body
                html: `
                        <center>
                            <img
                                width="100" height="100" src="https://firebasestorage.googleapis.com/v0/b/poltracker-7c3d0.appspot.com/o/logo.svg?alt=media&token=e79340c5-fbd4-4a91-a399-6712fca1b334"
                                style="display: block; margin-left: auto; margin-right: auto"
                            />
                        </center>
                        <h2>Verificar nou Usuari</h2>
                        <p>Hola ${user.displayName},</p>
                        <p>Segueix aquest enllaç per verificar la teva adreça electrònica.</p>
                        <div style="border-bottom: 1px solid; text-align: center; margin: 25px">
                            <p>
                                <a
                                    href=${link}
                                    style="
                                        align-items: center;
                                        background: #1343de;
                                        border-radius: 9999px;
                                        border: 0;
                                        color: #fff;
                                        cursor: pointer;
                                        font-weight: 600;
                                        padding: 8px 24px;
                                        transition: opacity 0.2s ease;
                                    "
                                >Verifica correu electrònic</a>
                            </p>
                        </div>                            
                        <p>Si no has demanat verificar aquesta adreça, pots ignorar aquest correu.</p>
                        <p>Gràcies,</p>
                        <p>L'equip de: Poltracker</p>`,
            })
            .then(console.log("Message with link sent to: %s", user.email));
    }
    main().catch(console.error);
};
