/**
 * Crea un usuari nou a Firebase i a Mongo
 * Si falla algun dels dos fa RollBack i borra tot
 * També envia un correu de verificació a l'adreça indicada.
 * L'enviament del correu no influeix, es fa en asincron, si falla
 * aquest enviament es grava l'usuari igualment, es pot demanar tornar a
 * enviar si s'entra sense estar verificat.
 */

import { firebaseAdmin } from "../../../utils/auth/firebaseAdmin";
import Account from "../../../models/Account";
import Entity from "../../../models/Entity";
import { connectToDatabase } from "../../../utils/db/mongodb";

const { EMAIL_USER, EMAIL_PASS } = process.env;


export const createUserInFirebase = ({ email, password , displayName  }) => {
    return new Promise((resolve) => {
        firebaseAdmin
            .auth()
            .createUser({
                email: email,
                emailVerified: false,
                password: password,
                displayName: displayName,
                disabled: false,
            })
            .then((user) => {
                //resolve User
                resolve(user);
            })
            .catch(() => resolve(false));
    });
};

const createUser = async (req, res) => {
    //Connect mongoose to DB
    await connectToDatabase();

    
    const createUserInMongoDB = ({ uid: _id }) => {
        return new Promise((resolve) => {
            if (_id) {
                let entity = new Entity(req.body); //Schema Mongoose
                console.log(entity)
                entity
                    .save()
                    .then((ent) => {
                        let account = new Account(req.body); //Schema Mongoose                        
                        account.idEntity = ent._id;
                        account._id = _id; //Same to Firebase
                        account
                            .save()
                            .then((user) => resolve({ user: user._id }))
                            .catch((e) => {
                                console.log(e)
                                resolve(false)});
                    })
                    .catch((e) => {
                        console.log(e)
                        resolve(false)});
            } else {                
                resolve(false);
            }
        });
    };

    //Rollback usuari de Firebase si no es crea a MongoDB
    const rollBackFirebase = ({ uid }) => {
        firebaseAdmin
            .auth()
            .deleteUser(uid)
            .then(() => {
                console.log("Successfully deleted user by Firebase");
            })
            .catch((error) => {
                console.log("Error deleting user firebase:", error);
            });
    };

    //Generar i enviar un link per a verificar el nou usuari
    const createLinkVerificationEmail = (user) => {
        return new Promise((resolve) => {
            // Admin SDK API to generate the email verification link.
            const useremail = user.email;
            firebaseAdmin
                .auth()
                .generateEmailVerificationLink(useremail /*actionCodeSettings*/)
                .then((link) => {
                    resolve(link);
                })
                .catch((error) => {
                    resolve(false);
                });
        });
    };

    //Enviar el email amb el link per verificar generat
    const sendVerificationEmail = (user, link) => {
        "use strict";
        const nodemailer = require("nodemailer");

        // async..await is not allowed in global scope, must use a wrapper
        async function main() {
            // create reusable transporter object using the default SMTP transport
            const transporter = nodemailer.createTransport({
                host: "smtp.servidor-correo.net",
                port: 587,
                secure: false, // true for 465, false for other ports
                auth: {
                    user: EMAIL_USER,
                    pass: EMAIL_PASS,
                },
            });

            // send mail with defined transport object
            transporter
                .sendMail({
                    from: '"Poltracker" <support@poltracker.es>', // sender address
                    to: user.email, // list of receivers
                    subject:
                        "Verifica l'adreça electrònica de l'aplicació Poltracker", // Subject line
                    //text: "Hello world?", // plain text body
                    html: `
                            <center>
                                <img
                                    width="100" height="100" src="https://firebasestorage.googleapis.com/v0/b/poltracker-7c3d0.appspot.com/o/logo.svg?alt=media&token=e79340c5-fbd4-4a91-a399-6712fca1b334"
                                    style="display: block; margin-left: auto; margin-right: auto"
                                />
                            </center>
                            <h2>Verificar nou Usuari</h2>
                            <p>Hola ${user.displayName},</p>
                            <p>Segueix aquest enllaç per verificar la teva adreça electrònica.</p>
                            <div style="border-bottom: 1px solid; text-align: center; margin: 25px">
                                <p>
                                    <a
                                        href=${link}
                                        style="
                                            align-items: center;
                                            background: #1343de;
                                            border-radius: 9999px;
                                            border: 0;
                                            color: #fff;
                                            cursor: pointer;
                                            font-weight: 600;
                                            padding: 8px 24px;
                                            transition: opacity 0.2s ease;
                                        "
                                    >Verifica correu electrònic</a>
                                </p>
                            </div>                            
                            <p>Si no has demanat verificar aquesta adreça, pots ignorar aquest correu.</p>
                            <p>Gràcies,</p>
                            <p>L'equip de: Poltracker</p>`,
                })
                .then(console.log("Message with link sent to: %s", user.email));
        }

        main().catch(console.error);
    };

    //Primer crear l'usuari a Firebase
    const userFirebase = await createUserInFirebase(req.body);
    console.log(`User created in Firebase: ${!!userFirebase?.uid}`);

    //Segon crea l'usuari a Mongo amb idAccount de Firebase
    //Si no hi ha user de Firebase o falla mongo resultat es false
    const rMongo = await createUserInMongoDB(userFirebase);
    console.log(`User created in MongoDB: ${!!rMongo}`);

    if (rMongo) {
        //Send Email
        //Create link verify email
        createLinkVerificationEmail(userFirebase)
            //Send Verify email
            .then((link) => sendVerificationEmail(userFirebase, link))
            .catch((err) => {
                console.log(err);
                console.log(
                    "El link per verificar l'email no s'ha pogut generar"
                );
            });
        //Si Mongo true User final OK
        res.status(200).json(rMongo);
        console.log(`user created: ${rMongo?.user}`);
    } else {
        //Si hi ha usuari de Firebase que el borri i doni error
        userFirebase && rollBackFirebase(userFirebase);
        res.status(200).send({ user: false });
    }
};

export default createUser;
